/*
 * Copyright (c) 2010, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/io.h>
#include <stdint.h>

#include "config.h"
#include "adc.h"

/* Mask to filter valid ADC channel numbers */
#define ADC_CHAN_MASK  0x03

/* Current value, updated every 10 ms */
static uint8_t adc_curval;

/* Initializes the ADC unit */
void adc_init(void)
{
	uint8_t adcsra;

	/* select ADC channel, VCC as reference and left-adjustment */
	ADMUX = (VOLT_ADC_CHAN & ADC_CHAN_MASK) | (1 << ADLAR);
	/* select ADC clock prescaler of 64 */
	adcsra = (1 << ADPS2) | (1 << ADPS1);
	/* enable ADC */
	adcsra |= (1 << ADEN);
	/* start initial conversion */
	adcsra |= (1 << ADSC);
	/* assign value to register */
	ADCSRA = adcsra;
	/* wait for initial conversion */
	while (ADCSRA & (1 << ADSC))
		;
}

/* Deinitializes the ADC unit */
void adc_deinit(void)
{
	/* Disable ADC to save power */
	ADCSRA &= ~(1 << ADEN);
}

/* Updates the current value buffer */
void adc_cyclic_10ms(void)
{
	if ((ADCSRA & (1 << ADSC)) == 0) {
		/* update buffer with current value */
		adc_curval = ADCH;
		/* start next conversion */
		ADCSRA |= (1 << ADSC);
	}
}

/* Returns the current value */
uint8_t adc_getval(void)
{
	return adc_curval;
}

