/*
 * Copyright (c) 2010, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/io.h>
#include <stdint.h>

#include "config.h"
#include "util.h"
#include "blinker.h"

/* Data structure for blinker */
typedef struct {
	unsigned phase   : 1;  // Current phase of blinker
	unsigned cycles  : 7;  // Cycles until next toggle
} blnk_state_t;

/* Instance of blinker data */
static blnk_state_t blnk_state;

/* Starts the blinker with a 'high' phase */
void blnk_start(void)
{
	blnk_state.phase = 1;
	blnk_state.cycles = LED_TIME_ON;
	set_pin(LED_PIN, LED_LEVEL_ON);
}

/* Stops the blinker */
void blnk_stop(void)
{
	set_pin(LED_PIN, LED_LEVEL_OFF);
}

/* Main blinker function, called every 100 ms */
void blnk_cyclic_100ms(void)
{
	/* decrement counter */
	if (blnk_state.cycles > 0)
		blnk_state.cycles--;
	if (blnk_state.cycles == 0) {
		/* counter reached zero, toggle LED state */
		if (blnk_state.phase) {
			/* turn LED off */
			set_pin(LED_PIN, LED_LEVEL_OFF);
			blnk_state.phase = 0;
			blnk_state.cycles = LED_TIME_OFF;
		} else {
			/* turn LED on */
			set_pin(LED_PIN, LED_LEVEL_ON);
			blnk_state.phase = 1;
			blnk_state.cycles = LED_TIME_ON;
		}
	}
}

/* Initializes the blinker */
void blnk_init(void)
{
	/* set port pin as output */
	DDRB |= (1 << LED_PIN);
	/* set pin to defined level */
	set_pin(LED_PIN, LED_LEVEL_OFF);
}

/* Deinitializes the blinker */
void blnk_deinit(void)
{
	/* set port pin as input */
	DDRB &= ~(1 << LED_PIN);
	/* disable pull-up to save power */
	PORTB &= ~(1 << LED_PIN);
}

