/*
 * Copyright (c) 2010, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef CONFIG_H
#define CONFIG_H

#include <avr/io.h>
#include "defs.h"

/* Maximum timer setting in seconds.
 * Use multiples of 256 for small values.
 * Range: 1 to 32767
 */
#define MAX_TIMER_SETTING   (256)

/* Status LED */
#define LED_PIN             PB2      // Pin LED is connected to
#define LED_LEVEL_ON        HIGH     // Pin level to turn LED on
#define LED_LEVEL_OFF       LOW      // Pin level to turn LED off
#define LED_TIME_ON         1        // On-time when blinking [100ms]
#define LED_TIME_OFF        9        // Off-time when blinking [100ms]

/* External control pin */
#define CTRL_PIN            PB4      // Pin used as external control
#define CTRL_LEVEL_STOPPED  LOW      // Pin level while timer is stopped
#define CTRL_LEVEL_RUNNING  HIGH     // Pin level while timer is running

/* Pushbuttons */
#define START_BUTTON_PIN    PB1      // Pin start button is connected to
#define STOP_BUTTON_PIN     PB0      // Pin stop button is connected to

/* Control voltage for setting timeout */
#define VOLT_ADC_CHAN       ADC3     // ADC channel for control voltage
#define VOLT_REVERSE        NO       // YES = period antiprop. to voltage

/* Sleep mode */
#define SLEEP_MODE_ENABLED  YES      // Power down device if idle
#define SLEEP_TIMEOUT       50       // Timeout until sleep when idle [100ms]
#define SLEEP_WAKEUP_PIN    PB1      // Pin used to wake up from sleep

#endif

