/*
 * Copyright (c) 2010, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "state.h"

#include "sleep.h"

#if (SLEEP_MODE_ENABLED == YES)

/* Sleep timeout counter */
static uint8_t slp_sleepy_cnt;

/* Dummy interrupt to wake up */
EMPTY_INTERRUPT(PCINT0_vect)

/* Cyclic function to decrement timeout counter */
void slp_cyclic_100ms(void)
{
	if (slp_sleepy_cnt > 0)
		slp_sleepy_cnt--;
	if (slp_sleepy_cnt == 0)
		st_enqueue_event(ST_E_SLEEPY);
}

/* Power down device */
void slp_powerdown(void)
{
	/* Configure the wakeup pin as input with pull-up activated
	 * to get clean pin changes when button is pressed.
	 */
	DDRB &= ~(1 << SLEEP_WAKEUP_PIN);
	PORTB |= (1 << SLEEP_WAKEUP_PIN);
	/* Activate pin change interrupt for this pin */
	PCMSK = (1 << SLEEP_WAKEUP_PIN);
	GIMSK |= (1 << PCIE);

	/* Enter power down mode. The call returns when 
	 * the interrupt has happened.
	 */
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sleep_mode();

	/* Deactivate further pin change interrupts */
	GIMSK &= ~(1 << PCIE);

	/* Send WAKEUP event to change to IDLE state */
	st_enqueue_event(ST_E_WAKEUP);
}

/* Restart timeout */
void slp_restart(void)
{
	slp_sleepy_cnt = SLEEP_TIMEOUT;
}

#endif

